{***********************************************************}
{                                                           }
{  FlatBtn  - A Windows 95/98 style tool bar button,        }
{             which is a TSpeedButton decendant.            }
{                                                           }
{  Copyright (C) 2000, by Spyder Mania Software Co.         }
{                      http://www.spydermania.com/          }
{  All rights reserved                                      }
{                                                           }
{  You may freely distribute, use and modify this code as   }
{  long as this statement and the above copyright           }
{  information are included and remain unmodified.          }
{                                                           }
{  I request any modifications to the source, any           }
{  suggestions, or comments to be emailed to me at:         }
{         spyder@spydermania.com                            }
{                                                           }
{  Feel free to use this component in any of your           }
{  applications.  All I ask is you give credit where credit }
{  is due. In other words, please give me credit in your    }
{  applications' help file, or about box.  Thank You!       }
{                                                           }
{***********************************************************}

unit FlatBtn;
{
  TFlatBtn is a descendant of TSpeedButton that overrides the
  default Painting and Mouse handling.

  TFlatBtn provides a new Flat property to decipher weather the
  button should be flat, or standard.  When Flat is true the
  button is flat with no border.  When the mouse cursor moves over
  the button, it rises, and when the mouse cursor leaves the button,
  it returns flat again.

  When Flat is false the button acts as a standard TSpeedButton.
  We override the normal drawing to give the button a Windows 95/98
  button appearance.

  The idea for this component comes from Windows 95/98 task bar
  buttons.  There doesn't seem to be any noticeable impact on
  performance, however, your mileage may vary.
}
interface

uses SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
     Forms, Dialogs, Buttons;

type
  TFlatBtn = class(TSpeedButton)

  private
    { Private declarations }
    FMouseOver: Boolean;
    FFlat: Boolean;
    procedure SetFlat(Value: Boolean);

  public
    { Public declarations }
    procedure PaintAsRaised;
    procedure repaint;
    constructor Create(AnOwner: TComponent); override;
    destructor Destroy; override;

  protected
    { Protected declarations }
    procedure Paint; override;
    procedure WMMouseMove(var Message: TWMMouseMove); message WM_MouseMove;

  published
    { Published declarations }
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property MouseOver: Boolean read FMouseOver write FMouseOver;
    property Flat: Boolean read FFlat write SetFlat default False;
  end;

procedure Register;

implementation

constructor TFlatBtn.Create;
begin
  inherited Create(AnOwner);
  { set defaults}
  Width := 25;
  Height := 25;
  MouseOver := False;

end;

destructor TFlatBtn.Destroy;
begin
  inherited Destroy;
end;

procedure TFlatBtn.SetFlat(Value: Boolean);
begin
  If Value <> FFlat then
  begin
    FFlat := Value;
    RePaint;
  end;
end;

procedure TFlatBtn.WMMouseMove(var Message: TWMMouseMove);
{This is the heart of the component.  Here we determine when the
 mouse cursor is over the button and when the mouse has left the
 buttons' client area.  Delphi 1 does not have an onMouseOut event,
 so here is a fairly generic way of doing it with onMouseMove.

 Feel free to use this in your components if you need to determine
 weather the mouse cursor is over your component or not.

 Do not forget to make the protected declaration for this procedure}

begin
  inherited;	{ perform default handling }
  If Flat Then
  Begin

{NOTE:  If you want your button to Raise and lower while your application is
        NOT active, just comment out 'and (GetParentForm(Self).Active)'
        from the following line.  See flatbtn.txt for more information.}
    If (NOT MouseCapture) and (GetParentForm(Self).Active) Then
    Begin
      SetCaptureControl(Self);
      If Enabled and Visible Then
      {The Mouse Cursor is over the button}
      Begin
        {Replace these two lines of code with code to perform
         what you need while the mouse cursor is over your control}
        MouseOver := True;
        PaintAsRaised;
      end;
    end;
    If MouseCapture and ((Message.xpos < 0) or (Message.ypos < 0) or
     (Message.xpos > Width) or (Message.Ypos > Height)) and MouseOver Then
    {The Mouse Cursor has left the button}
    Begin
    {Replace the following two lines only to perform what you need after the
     mouse cursor has left your control}
      MouseOver := False;
      RePaint;
      SetCaptureControl(Nil);{This line is required}
    end;
  end;
end;

procedure TFlatBtn.Paint;
begin
  inherited Paint;

  If Flat then
  Begin
    with Canvas do
    begin
      Pen.Color := clBtnFace;
      MoveTo(0, 0);
      LineTo(0, height);
      LineTo(Width, height);
      LineTo(Width, 0);
      LineTo(1, 0);
      LineTo(1, height - 1);
      LineTo(Width - 1, height - 1);
      LineTo(Width - 1, 1);
      LineTo(1, 1);
      MoveTo(1, height - 2);
      LineTo(Width - 2, height - 2);
      LineTo(Width - 2, 1);
      if FState in [bsDown, bsExclusive] then
      begin
        Pen.Color := clBtnHighlight;
        MoveTo(Width - 1, 0);
        LineTo(Width - 1, Height - 1);
        LineTo(-1, Height - 1);
        Pen.Color := clWindowFrame;
        MoveTo(0, Height - 1);
        LineTo(0, 0);
        LineTo(Width, 0);
        Pen.Color := clBtnShadow;
        MoveTo(1, Height - 2);
        LineTo(1, 1);
        LineTo(width - 1, 1);
      end;
    end;
  end
  else
  PaintAsRaised;
end;

procedure TFlatBtn.RePaint;
begin
  Invalidate;
end;

procedure TFlatBtn.PaintAsRaised;
begin
{This procedure paints the button.
 This procedure is called to Raise the button while the mouse
 is over it.  This procedure is also called when TFlatBtn.Flat is false
 to draw the button.

 Some of the drawing in this procedure was borrowed from Calmira VCLs'
 Style Speed Button originally written by Li-Hsin Huang, may have
 been modified by Erwin Dokter as he is the current programming
 coordinator of Calmira.  Visit http://www.calmira.org/

 The code in this procedure is not completely taken from
 Style Speed Button.  I modified it some to suit my needs.

 The reality is I created the component, and did not like what I had for the
 normal state (or raised state) of the button.  So I peeked a little bit at
 the code for painting TStyleSpeed in bsWin95 state.  Hope no one minds <G>
 }

   inherited Paint;
   with Canvas do
    begin
      Pen.Color := clBtnHighlight;
      MoveTo(0, Height - 1);
      LineTo(0, 0);
      LineTo(Width - 1, 0);
      Pen.Color := clBtnFace;
      MoveTo(1, Height - 2);
      LineTo(1, 1);
      LineTo(Width - 2, 1);
      Pixels[0, Height - 1] := clWindowFrame;
      Pixels[1, Height - 2] := clBtnShadow;
      if (FState in [bsDown, bsExclusive]) and (NOT Flat) then
      begin
        Pen.Color := clBtnHighlight;
        MoveTo(Width - 1, 0);
        LineTo(Width - 1, Height - 1);
        LineTo(-1, Height - 1);
        Pen.Color := clWindowFrame;
        MoveTo(0, Height - 1);
        LineTo(0, 0);
        LineTo(Width, 0);
        Pen.Color := clBtnShadow;
        MoveTo(1, Height - 2);
        LineTo(1, 1);
        LineTo(width - 1, 1);
      end;
  end;
end;

procedure Register;
begin
  RegisterComponents('Additional', [TFlatBtn]);
end;

end.
